export type TRgba = [number, number, number, number];
/**
 * ColorPalette
 *
 * Class for extracting a color palette.
 */
/**
 * A class for extracting color palettes from images.
 *
 * This class provides functionality to analyze images and extract their dominant colors,
 * taking into account color similarity and transparency.
 *
 * @remarks
 * The color similarity is calculated using the CIE76 formula, which provides a perceptual
 * difference between colors that approximates human vision.
 *
 * @example
 * ```typescript
 * const palette = await ColorPalette.getColors('path/to/image.jpg', 5, 10);
 * console.log(palette); // ['#FF0000', '#00FF00', '#0000FF', ...]
 * ```
 */
export declare class ColorPalette {
    /**
     * getColors
     *
     * Extracts a color palette from an image.
     *
     * @param imgSrc - The path to the image to extract the color palette from (base64 strings are also supported).
     *
     * @param colorsFromPalette - Number of colors to extract, default is 2.
     *
     * @param similarityThreshold - Threshold for color similarity based on the CIE76 formula.
     *
     * Color Similarity Reference Table:
     *
     * | Delta E Value | Perception                                |
     * |---------------|-------------------------------------------|
     * | <= 1.0        | Not perceptible to human eye              |
     * | 1.0 - 2.0     | Perceptible through close observation     |
     * | 2.0 - 10.0    | Perceptible at a glance                   |
     * | 11.0 - 49.0   | Colors are more similar than opposite     |
     * | 100           | Colors are exact opposites                |
     *
     * @returns Promise with an array of colors in hexadecimal format.
     */
    static getColors(imgSrc: string, colorsFromPalette?: number, similarityThreshold?: number): Promise<string[]>;
    /**
     * _countPixelColors
     *
     * Counts the occurrence of each color present in the pixels of an image.
     */
    private static _countPixelColors;
    /**
     * _sortColorsByCount
     *
     * Sorts the colors from an object containing color counts.
     */
    private static _sortColorsByCount;
    /**
     * _filterSimilarColors
     *
     * Filters out colors that are too similar to those already in the palette.
     */
    private static _filterSimilarColors;
    /**
     * _colorDifference
     *
     * Calculates the difference between two colors using the CIE76 formula.
     */
    private static _colorDifference;
    /**
     * _getRgbaColorString
     *
     * Returns a string representing the RGBA color.
     */
    private static _getRgbaColorString;
}
//# sourceMappingURL=image-palette-extractor.d.ts.map