"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorPalette = void 0;
// Utils
var ColorUtils_1 = require("../utils/ColorUtils");
/**
 * ColorPalette
 *
 * Class for extracting a color palette.
 */
/**
 * A class for extracting color palettes from images.
 *
 * This class provides functionality to analyze images and extract their dominant colors,
 * taking into account color similarity and transparency.
 *
 * @remarks
 * The color similarity is calculated using the CIE76 formula, which provides a perceptual
 * difference between colors that approximates human vision.
 *
 * @example
 * ```typescript
 * const palette = await ColorPalette.getColors('path/to/image.jpg', 5, 10);
 * console.log(palette); // ['#FF0000', '#00FF00', '#0000FF', ...]
 * ```
 */
var ColorPalette = /** @class */ (function () {
    function ColorPalette() {
    }
    /**
     * getColors
     *
     * Extracts a color palette from an image.
     *
     * @param imgSrc - The path to the image to extract the color palette from (base64 strings are also supported).
     *
     * @param colorsFromPalette - Number of colors to extract, default is 2.
     *
     * @param similarityThreshold - Threshold for color similarity based on the CIE76 formula.
     *
     * Color Similarity Reference Table:
     *
     * | Delta E Value | Perception                                |
     * |---------------|-------------------------------------------|
     * | <= 1.0        | Not perceptible to human eye              |
     * | 1.0 - 2.0     | Perceptible through close observation     |
     * | 2.0 - 10.0    | Perceptible at a glance                   |
     * | 11.0 - 49.0   | Colors are more similar than opposite     |
     * | 100           | Colors are exact opposites                |
     *
     * @returns Promise with an array of colors in hexadecimal format.
     */
    ColorPalette.getColors = function (imgSrc, colorsFromPalette, similarityThreshold // Default to "perceptible at a glance"
    ) {
        var _this = this;
        if (colorsFromPalette === void 0) { colorsFromPalette = 2; }
        if (similarityThreshold === void 0) { similarityThreshold = 10; }
        var img = new Image();
        img.src = imgSrc;
        return new Promise(function (resolve, reject) {
            img.onload = function () {
                var canvas = document.createElement('canvas');
                var context = canvas.getContext('2d');
                canvas.width = img.width;
                canvas.height = img.height;
                context.drawImage(img, 0, 0);
                var imageData = context.getImageData(0, 0, canvas.width, canvas.height);
                var pixelData = imageData.data;
                var colorCounts = _this._countPixelColors(pixelData);
                var sortedColors = _this._sortColorsByCount(colorCounts);
                // Filter out similar colors based on the threshold
                var distinctColors = _this._filterSimilarColors(sortedColors, similarityThreshold).slice(0, colorsFromPalette);
                // Build the final palette in hexadecimal format
                var colorPalette = distinctColors.map(function (color) {
                    return ColorUtils_1.rgbaToHex.apply(void 0, (0, ColorUtils_1.extractRgbAsTuple)(_this._getRgbaColorString(color)));
                });
                resolve(colorPalette);
            };
            img.onerror = function (error) {
                return reject("An error occurred while loading the image: Error ".concat(error));
            };
        });
    };
    /**
     * _countPixelColors
     *
     * Counts the occurrence of each color present in the pixels of an image.
     */
    ColorPalette._countPixelColors = function (pixelData) {
        var colorCounts = {};
        for (var i = 0; i < pixelData.length; i += 4) {
            var r = pixelData[i];
            var g = pixelData[i + 1];
            var b = pixelData[i + 2];
            var a = pixelData[i + 3];
            var colorKey = [r, g, b, a].join(',');
            // Excludes white and possible transparent pixels from extraction
            if (a === 0 || [r, g, b].every(function (value) { return value === 255; })) {
                continue;
            }
            if (colorKey in colorCounts) {
                colorCounts[colorKey]++;
            }
            else {
                colorCounts[colorKey] = 1;
            }
        }
        return colorCounts;
    };
    /**
     * _sortColorsByCount
     *
     * Sorts the colors from an object containing color counts.
     */
    ColorPalette._sortColorsByCount = function (colorCounts) {
        var sortedColors = Object.keys(colorCounts).sort(function (a, b) { return colorCounts[b] - colorCounts[a]; });
        return sortedColors.map(function (colorKey) { return colorKey.split(',').map(function (c) { return parseInt(c); }); });
    };
    /**
     * _filterSimilarColors
     *
     * Filters out colors that are too similar to those already in the palette.
     */
    ColorPalette._filterSimilarColors = function (colors, threshold) {
        var _this = this;
        var distinctColors = [];
        var _loop_1 = function (color) {
            var shouldPushColor = distinctColors.length === 0 ||
                !distinctColors.some(function (existingColor) { return _this._colorDifference(color, existingColor) < threshold; });
            if (shouldPushColor) {
                distinctColors.push(color);
            }
        };
        for (var _i = 0, colors_1 = colors; _i < colors_1.length; _i++) {
            var color = colors_1[_i];
            _loop_1(color);
        }
        return distinctColors;
    };
    /**
     * _colorDifference
     *
     * Calculates the difference between two colors using the CIE76 formula.
     */
    ColorPalette._colorDifference = function (color1, color2) {
        var r1 = color1[0], g1 = color1[1], b1 = color1[2];
        var r2 = color2[0], g2 = color2[1], b2 = color2[2];
        // Convert RGB to Lab color space
        var lab1 = (0, ColorUtils_1.rgbToLab)(r1, g1, b1);
        var lab2 = (0, ColorUtils_1.rgbToLab)(r2, g2, b2);
        // Calculate CIE76 color difference
        var deltaL = lab1[0] - lab2[0];
        var deltaA = lab1[1] - lab2[1];
        var deltaB = lab1[2] - lab2[2];
        return Math.sqrt(deltaL * deltaL + deltaA * deltaA + deltaB * deltaB);
    };
    /**
     * _getRgbaColorString
     *
     * Returns a string representing the RGBA color.
     */
    ColorPalette._getRgbaColorString = function (rgbaArray) {
        var r = rgbaArray[0], g = rgbaArray[1], b = rgbaArray[2], a = rgbaArray[3];
        return "rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(a / 255, ")");
    };
    return ColorPalette;
}());
exports.ColorPalette = ColorPalette;
//# sourceMappingURL=image-palette-extractor.js.map